/*
 * Proyecto MiCasa, version 1.
 * Copyright (C) 2002 Grzegorz Adam Hankiewicz <gradha@terra.es> y
 * Jon Lpez de Dicastillo <darwater@hotmail.com>.
 * Curso 2001-2002
 * Facultad de Ingeniera
 * Universidad de Deusto
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package MiCasa;

import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.util.Vector;

/**
 * Esta clase es una interfaz entre el programa principal (servidor)
 * y la base de datos SQL proporcionada por McKoi. Realiza peticiones
 * transformando los valores de java en peticiones de SQL y viceversa.
 */
public class BaseDatos
{
   /** Conexin permanente con la base de datos */
   private static Connection conexionBD;

   /** Parmetro de conexin con la base de datos */
   private final static String userBD = "usuario";
   
   /** Parmetro de conexin con la base de datos */
   private final static String passBD = "f1and1t0";
   
   /** URL donde se almacena la configuracin de McKoi */
   private final static String urlBD =
      "jdbc:mckoi:local://MckoiSQLDatabase/BaseDatos.conf";
      
   /**
    * Verifica que el nombre y la password estn en la base de datos de
    * personas autorizadas.
    * @return Falso si se deniega el acceso.
    */
   public static boolean UsuarioTienePermiso(String nombre, String password) {
      boolean valido = false;
      try {
         Statement p = conexionBD.createStatement();
         ResultSet r;

         r = p.executeQuery("SELECT Usuarios.nombre, Usuarios.password " +
            "FROM Usuarios WHERE Usuarios.nombre = '" + nombre + "'");
         if(r.next()) {
            if(password.equals(r.getString("password")))
               valido = true;
            else
               System.err.println("Password incorrecto para " + nombre);
         }
         else
            System.err.println(nombre + " - " + password + " no existe en BD");

         p.close();
      }
      catch(SQLException e) {
         System.err.println("Fallo SQL. Razn: " + e.getMessage());
      }
      return valido;
   }

   /**
    * Funcin que intenta hacer una lectura rpida de ciertos valores
    * de la base de datos. Esta lectura se usa para saber si la base de
    * datos existe o es accesible.
    * @return False si hubo algn problema.
    */
   public static boolean LecturaPrevia() {
      System.out.println("...conectando...");
      // Intentamos establecer la conexin con la base de datos...
      Connection con;
      try {
         con = DriverManager.getConnection(urlBD, userBD, passBD);
      }
      catch(SQLException e) {
         System.out.println("Imposible conectar con la base de datos.\n" +
            "Razn: " + e.getMessage());
         return false;
      }

      try {
         Statement p = con.createStatement(); // Peticiones
         ResultSet r; // Resultado de las peticiones

         // Cuntas columnas hay en la tabla "Usuarios"?
         r = p.executeQuery("SELECT COUNT(*) FROM Usuarios");
         r.next();
         System.out.print(r.getInt(1) + " usuarios registrados: ");

         // Listar sus nombres
         r = p.executeQuery("SELECT nombre FROM Usuarios ORDER BY nombre");
         while(r.next()) System.out.print(r.getString(1) + "\t");
         System.out.println();

         // Cuantas viviendas hay registradas?
         r = p.executeQuery("SELECT COUNT(*) FROM Viviendas");
         r.next();
         System.out.println(r.getInt(1) + " viviendas en base de datos.");

         // Listado de viviendas
         r = p.executeQuery("SELECT * FROM Viviendas ORDER BY codigo");
         while(r.next()) System.out.println(LineaListadoVentas(r));

         r.close();
         p.close();
         con.close();
      }
      catch(SQLException e) {
         System.out.println("Error durante lecturas previas a BD\n" +
            "Mensaje SQLException: " + e.getMessage());
         try { con.close(); } catch(SQLException i) {};
         return false;
      }
      return true;
   }

   /**
    * Genera las tablas (casi) vacas de la base de datos. Se invoca
    * nicamente si una lectura anterior ha resultado imposible.
    */
   public static void GenerarVacia() {
      System.out.println("Voy a intentar regenerar la base de datos...");
      Connection con = null;
      String url = urlBD + "?create=true";
      try {
         con = DriverManager.getConnection(url, userBD, passBD);

         // Creamos las tablas...
         Statement p = con.createStatement();
         // Tabla de usuarios
         p.executeQuery("CREATE TABLE Usuarios ( " +
            "nombre        VARCHAR(100) NOT NULL, " +
            "password      VARCHAR(100) NOT NULL) ");

         p.executeUpdate("INSERT INTO Usuarios (nombre, password) VALUES " +
            "('root',      'supermacho') ," +
            "('1grhanki',  'yo') ," +
            "('darwater',  'hotmail')");

         // Tabla de viviendas
         p.executeQuery("CREATE TABLE Viviendas ( " +
            "codigo        INTEGER NOT NULL, " +
            "calle         VARCHAR(50) NOT NULL, " +
            "localidad     VARCHAR(20), " +
            "precio        INTEGER NOT NULL, " +   // aadir un contraint para > 0 ?
            "comprada      BOOLEAN DEFAULT FALSE)");

         p.executeUpdate("INSERT INTO Viviendas " +
            "(codigo, calle, localidad, precio) VALUES " +
            "('12',  'Castilla la Mancha 5 2',    'Baracaldo',   '250000')," +
            "('32',  'Juan de Garay 4 1',         'Baracaldo',   '180000')," +
            "('4',   'Gabriel Aresti 12 1D',      'Bilbao',      '150000')," +
            "('45',  'Ramn y Cajal 4',            'Madrid',      '321000')," +
            "('33',  'Guayana de Rojo 87',         'Irun',        '248200')," +
            "('18',  'Percebe Rue 1',              'Salazar',     '931559')," +
            "('9',   'Milochilco Dabo 3',         'Isuskiza',    '225000')");
      }
      catch(SQLException e) {
         System.out.println("No se pudo generar la base de datos.\n" +
            "Razn: " + e.getMessage());
      }

      // Cerramos la conexin
      try { con.close(); }
      catch(SQLException e) { e.printStackTrace(System.err); }
   }

   /**
    * Abre la conexin con el servidor SQL. Esto es una conexin
    * global que ser usada/compartida por todos los hilos.
    * @return False si no se pudo crear.
    */
   public static boolean AbrirConexion() {
      try {
         conexionBD = DriverManager.getConnection(urlBD, userBD, passBD);
         return true;
      }
      catch(SQLException e) {
         System.out.println("Imposible conectar con la base de datos.\n" +
            "Razn: " + e.getMessage());
         return false;
      }
   }

   /**
    * Busca en la base de datos todas las viviendas que coincidan con
    * el patrn especificado.
    * @param patron Es una cadena de texto con expresiones regulares del
    *               tipo '*' para muchos carcteres, y '?' para uno solo.
    * @return Vector de String por vivienda.
    */
   public static Vector ListadoViviendas(String patron) {
      Vector lista = new Vector();
      // Adaptamos el patrn a el patrn SQL
      patron = patron.toUpperCase();
      patron = patron.replace('*', '%');
      patron = patron.replace('?', '_');
      try {
         Statement p = conexionBD.createStatement(); // Peticiones
         ResultSet r; // Resultado de las peticiones

         // Listado de viviendas
         r = p.executeQuery("SELECT * FROM Viviendas " +
            "WHERE UPPER(calle) LIKE '" + patron + "' OR UPPER(localidad) " +
            "LIKE '" + patron + "' ORDER BY codigo");
         while(r.next()) lista.add(LineaListadoVentas(r));

         r.close();
         p.close();
      }
      catch(SQLException e) {
         System.out.println("Error en acceso a BD\n" +
            "Mensaje SQLException: " + e.getMessage());
      }
      return lista;
   }

   /**
    * Busca en la base de datos todas las viviendas.
    * @return Vector de String por vivienda.
    */
   public static Vector ListadoViviendas() {
      Vector lista = new Vector();
      try {
         Statement p = conexionBD.createStatement(); // Peticiones
         ResultSet r; // Resultado de las peticiones

         // Listado de viviendas
         r = p.executeQuery("SELECT * FROM Viviendas ORDER BY codigo");
         while(r.next()) lista.add(LineaListadoVentas(r));

         r.close();
         p.close();
      }
      catch(SQLException e) {
         System.out.println("Error en acceso a BD\n" +
            "Mensaje SQLException: " + e.getMessage());
      }
      return lista;
   }

   /**
    * Ayudante de las versiones de ListadoViviendas.
    * @param r El resultado de una bsqueda SQL en la base de datos. Debe
    *          contener una lnea vlida de la tabla Ventas.
    * @return Una lnea
    * de texto "tipo" construda a partir de los resultados de una select
    * de la base de datos contra la tabla Viviendas.
    */
   private static String LineaListadoVentas(ResultSet r) throws SQLException {
      String cadena = new String(r.getInt("codigo") + " \"" +
         r.getString("calle") + "\" \"" +
         r.getString("localidad") + "\" " +
         r.getInt("precio"));
      if(r.getBoolean("comprada"))  return cadena.concat(" ya comprada.");
      else                          return cadena.concat(" en venta.");
   }

   /**
    * Usado para obtener el precio de venta de una vivienda.
    * @return El precio (>0) si la vivienda existe y se puede comprar, 0 si
    * la vivienda est comprada, y negativo si el cdigo de vivienda
    * no se encuentra en la base de datos.
    */
   public static int ComprarVivienda(int numVivienda) {
      try {
         Statement p = conexionBD.createStatement(); // Peticiones
         ResultSet r; // Resultado de las peticiones

         // Buscamos la vivienda indicada
         r = p.executeQuery("SELECT * FROM Viviendas WHERE codigo = '" +
            numVivienda + "'");
         int precio = -1;
         if(!r.next())
            return -1;
         if(r.getBoolean("comprada"))
            return 0;
         precio = r.getInt("precio");
         r.close();
         p.close();
         return precio;
      }
      catch(SQLException e) {
         System.out.println("Error en acceso a BD\n" +
            "Mensaje SQLException: " + e.getMessage());
      }
      return -1;
   }

   /**
    * Modifica la base de datos actualizando el estado de compra de la
    * vivienda a true.
    */
   public static void ConfirmarCompra(int numVivienda) {
      try {
         Statement p = conexionBD.createStatement(); // Peticiones
         // Marcamos la vivienda indicada como comprada
         p.executeUpdate("UPDATE Viviendas SET comprada='1' " +
            "WHERE codigo='" + numVivienda + "'");
         p.close();
      }
      catch(SQLException e) {
         System.out.println("Error en acceso a BD\n" +
            "Mensaje SQLException: " + e.getMessage());
      }
   }
}

