/*
 * Proyecto MiCasa, version 1.
 * Copyright (C) 2002 Grzegorz Adam Hankiewicz <gradha@terra.es> y
 * Jon Lpez de Dicastillo <darwater@hotmail.com>.
 * Curso 2001-2002
 * Facultad de Ingeniera
 * Universidad de Deusto
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package MiCasa;

import java.net.Socket;
import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.FileWriter;
import java.lang.Integer;
import java.util.Vector;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;

/**
 * Clase intermedia entre el programa cliente y las conexiones por socket
 * con el servidor. La comunicacin se realiza mediante comandos de texto
 * por un nico canal.
 */
public class ConexionCliente extends Thread
{
   /** Socket a travs del cul llevar a cabo la comunicacin. */
	private Socket so;

   /** Filtro para poder leer del socket lnea a lnea. */
	private BufferedReader br;

   /** Filtro para poder escribir al socket lneas enteras. */
	private DataOutputStream dos;

   /**
    * Establece la conexin. Requiere la direccin IP o nmbre de la mquina
    * y el puerto en el cual se va a realizar la conexin.
    */
   public ConexionCliente(String host, String puerto) throws IOException {
      so = new Socket(host, Integer.parseInt(puerto));
      br = new BufferedReader(new InputStreamReader(so.getInputStream()));
      dos = new DataOutputStream(so.getOutputStream());
   }

   /**
    * Manda el mensaje de salida al servidor y cierra las conexiones.
    */
   public void Desconectar() {
      try {
         dos.writeBytes("SALIR\r\n");
         dos.close();
         br.close();
         so.close();
      }
      catch(IOException ioe) {
         System.err.println(ioe);
      }
   }

   /**
    * Intenta realizar una autentificacin con el servidor para iniciar el
    * dilogo.
    * @return False si el servidor rechaz la peticin.
    */
   public boolean Autentificar(String usuario, String clave) throws Exception {
      dos.writeBytes("USER " + usuario + "\r\n");
      String respuesta = br.readLine();
      if(!respuesta.startsWith("200 OK"))
         return false;
      dos.writeBytes("PASSWORD " + clave + "\r\n");
      respuesta = br.readLine();
      if(respuesta.startsWith("201 OK"))
         return true;
      return false;
   }

   /**
    * Pide al servidor que enve un listado de viviendas.
    * @param patron Puede ser NULL para solicitar todas las viviendas,
    * o una expresin regular que use '*' y '?'.
    * @return Vector de String por vivienda.
    */
   public Vector ListadoViviendas(String patron) {
      try {
         dos.writeBytes("LISTADO_VIVIENDAS");
         if(patron != null) dos.writeBytes(" " + patron);
         dos.writeBytes("\r\n");
         Vector lista = new Vector();
         String respuesta;
         do {
            respuesta = br.readLine();
            if(respuesta.startsWith("4")) { // Esto significa error grave
               System.err.println("Error grave: " + respuesta);
               break;
            }
            if(respuesta.startsWith("ELEM "))
               lista.add(respuesta.substring(5));
         } while (respuesta.startsWith("202 FINLISTA") == false);
         return lista;
      }
      catch(IOException e) {
         System.err.println(e);
         return new Vector();
      }
   }

   /**
    * Realiza una peticin de compra al servidor.
    * @return Negativo si la casa no existe, cero si est comprada, o el
    * precio positivo de la vivienda si se puede comprar.
    */
   public int ComprarVivienda(String cadena) {
      try {
         StringTokenizer sTok = new StringTokenizer(cadena, " ");
         int numero = Integer.parseInt(sTok.nextToken());
         dos.writeBytes("COMPRAR_VIVIENDA " + numero + "\r\n");
         String respuesta = br.readLine();
         if(respuesta.startsWith("405 ERR "))
            return 0;
         else if(respuesta.startsWith("203 OK ")) {
            sTok = new StringTokenizer(respuesta.substring(7), " ");
            return Integer.parseInt(sTok.nextToken());
         }
      }
      catch(Exception e) {
         System.err.println(e);
      }
      return -1;
   }

   /**
    * Tras ComprarVivienda, si su valor de respuesta fue mayor que cero,
    * entonces hay que confirmar o rechazar la compra para que el servidor
    * salga de su estado de bloqueo.
    */
   public void ConfirmarCompra(boolean respuesta) {
      try {
         if(respuesta)
            dos.writeBytes("CONFIRMAR_COMPRA\r\n");
         else
            dos.writeBytes("RECHAZAR_COMPRA\r\n");
         br.readLine(); // Hay que limpiar el buffer de respuesta
      }
      catch(Exception e) {
         System.err.println(e);
      }
   }
}
