/*
 * Proyecto MiCasa, version 1.
 * Copyright (C) 2002 Grzegorz Adam Hankiewicz <gradha@terra.es> y
 * Jon Lpez de Dicastillo <darwater@hotmail.com>.
 * Curso 2001-2002
 * Facultad de Ingeniera
 * Universidad de Deusto
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package MiCasa;

import java.net.Socket;
import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.File;
import java.io.FileReader;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;
import java.util.Vector;
import java.lang.Integer;

/**
 * La clase ServidorCasa implementa la funcionalidad del servidor.
 * Es capaz de recibir y procesar los comandos que le enva el
 * cliente a travs del socket, y los transforma en peticiones sobre
 * la base de datos gracias a la clase {@link MiCasa.BaseDatos}.
 */
public class ServidorCasa extends Thread
{
   /** Nombre del usuario que ha entrado. */
   private String usuario;
   
   /** Socket a travs del cual llevar a cabo la comunicacin. */
   private Socket so;

   /** Filtro para poder leer del socket lnea a lnea. */
   private BufferedReader br;

   /** Filtro para poder escribir al socket lneas enteras. */
   private DataOutputStream dos;

   /** Variable con el estado del servidor. */
   private int estado;

   /** Variable con la vivienda que se desea comprar */
   private int compraVivienda;

   /**
    * Constructor del servidor.
    * @param s Socket a travs del cual se lleva la comunicacin.
    */
   public ServidorCasa(Socket s) {
      so = s;
      try {
         br = new BufferedReader(new InputStreamReader(so.getInputStream()));
         dos = new DataOutputStream(so.getOutputStream());
         estado = 0;
      }
      catch(IOException e) {
         System.err.println("ServidorCasa.constructor: " + e);
      }
   }

   /**
    * Cierra una conexin, tanto el socket como los filtros.
    */
   private void Desconectar() throws IOException {
      // Cerramos los filtros y el socket
      br.close();
      dos.close();
      so.close();
      estado = 69;
   }

   /**
    * Lleva a cabo el proceso de recepcin y tratamiento de comandos remotos.
    */
   public void run() {
      while(estado != 69) {
         try {
            String linea=br.readLine();

            // Si el extremo opuesto ha cerrado la comunicacin
            if(linea == null) return;

            System.out.println("Recibido desde: " +
               so.getInetAddress().getHostAddress() + " --> " + linea);

            StringTokenizer sTok = new StringTokenizer(linea," ");
            String c = sTok.nextToken().toUpperCase();
            if(c.equals("SALIR")) TratarSALIR();
            switch(estado) {
               case 0: // Esperando nombre
                  if(c.equals("USER")) TratarUSER(sTok);
                  else c = null;
                  break;

               case 1: // Esperando password
                  if(c.equals("USER")) TratarUSER(sTok);
                  else if(c.equals("PASSWORD")) TratarPASSWORD(sTok);
                  else c = null;
                  break;

               case 2: // Esperando peticin normal
                  if(c.equals("LISTADO_VIVIENDAS"))
                     TratarLISTADO_VIVIENDAS(sTok);
                  else if(c.equals("COMPRAR_VIVIENDA"))
                     TratarCOMPRAR_VIVIENDA(sTok);
                  else c = null;
                  break;

               case 3: // Esperando confirmacin compra
                  if(c.equals("RECHAZAR_COMPRA"))
                     TratarRECHAZAR_COMPRA();
                  else if(c.equals("CONFIRMAR_COMPRA"))
                     TratarCONFIRMAR_COMPRA();
                  else c = null;
                  break;
            }
            if(c == null)
               dos.writeBytes("469 ERR COMANDO NO RECONOCIDO O NO ADECUADO.\r\n");
         }
         catch(IOException e) {
            System.err.println("ServidorCasa.run " + e);
            break;
         }
         catch(NoSuchElementException e) {}
      }
   }

   /**
    * Trata el mensaje USER del usuario. Si el usuario ha proporcionado un
    * nombre vlido, se acepta y se cambia de estado.
    */
   private void TratarUSER(StringTokenizer sTok) throws IOException {
      try {
         usuario = sTok.nextToken();
         dos.writeBytes("200 OK Bienvenido " + usuario + ".\r\n");
         estado = 1;
      }
      catch(NoSuchElementException e) {
         dos.writeBytes("400 ERR Falta el nombre del usuario.\r\n");
      }
   }

   /**
    * Trata el mensaje PASSWORD del usuario. Aparte de verificar si se ha
    * proporcionado una clave, hay que verificar en la base de datos si el
    * usuario anteriormente introducido es vlido con esta clave.
    */
   private void TratarPASSWORD(StringTokenizer sTok) throws IOException {
      try {
         if(BaseDatos.UsuarioTienePermiso(usuario, sTok.nextToken())) {
            dos.writeBytes("201 OK Bienvenido al sistema.\r\n");
            estado = 2;
         }
         else {
            dos.writeBytes("401 ERR La clave es incorrecta.\r\n");
            estado = 0;
         }
      }
      catch(NoSuchElementException e) {
         dos.writeBytes("402 ERR Falta la clave.\r\n");
         estado = 0;
      }
   }

   /**
    * Trata el mensaje LISTADO_VIVIENDAS. Bsicamente vuelca el resultado
    * de una bsqueda en la base de datos, de todos los elementos o
    * nicamente aquellos indicados por un patrn.
    */
   private void TratarLISTADO_VIVIENDAS(StringTokenizer sTok) throws IOException {
      Vector lista = null;
      try {
         lista = BaseDatos.ListadoViviendas(sTok.nextToken());
      }
      catch(NoSuchElementException e) {
         lista = BaseDatos.ListadoViviendas();
      }

      for(int f = 0; f < lista.size(); f++)
         dos.writeBytes("ELEM " + lista.get(f) + "\r\n");
      dos.writeBytes("202 FINLISTA\r\n");
   }

   /**
    * Trata el mensaje COMPRAR_VIVIENDA. El usuario introduce un cdigo
    * numrico segn el cdigo de la vivienda de la base de datos, y en
    * caso de que se pueda comprar, cambia de estado esperando la confirmacin
    * tras enviar un mensaje de xito.
    */
   private void TratarCOMPRAR_VIVIENDA(StringTokenizer sTok) throws IOException {
      try {
         compraVivienda = Integer.parseInt(sTok.nextToken());
         int precio = BaseDatos.ComprarVivienda(compraVivienda);
         if(precio > 0) {
            dos.writeBytes("203 OK " + precio + " Confirmar precio.\r\n");
            estado = 3;
         }
         else if(precio == 0)
            dos.writeBytes("405 ERR Vivienda ya comprada.\r\n");
         else
            dos.writeBytes("403 ERR Cdigo no existe.\r\n");
      }
      catch(NoSuchElementException e) {
         dos.writeBytes("406 ERR Falta el cdigo de vivienda.\r\n");
      }
   }

   /**
    * El usuario aborta la compra. Pues bueno, volvemos al estado anterior.
    */
   private void TratarRECHAZAR_COMPRA() throws IOException {
      dos.writeBytes("205 OK Compra rechazada.\r\n");
      estado = 2;
   }

   /**
    * Compra confirmada. Hay que modificar en la base de datos este dato.
    */
   private void TratarCONFIRMAR_COMPRA() throws IOException {
      BaseDatos.ConfirmarCompra(compraVivienda);
      dos.writeBytes("204 OK Compra confirmada.\r\n");
      estado = 2;
   }

   /**
    * Manda un mensaje de despedida antes de cerrar el grifo.
    */
   private void TratarSALIR() {
      try {
         dos.writeBytes("207 OK Adios.\r\n");
         this.Desconectar();
      }
      catch(Exception e) {
      }
   }
}

